#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat May 18 23:20:02 2019

@author: dorgham
"""

import networkx as nx
import codecs
import random
import statistics

G = nx.Graph()

fb15k_train_file_path = 'ConvE/train.txt'
fb15k_test_file_path = 'ConvE/test.txt'
path2vec_dataset_fpath = 'FB15k237_path2vec_ready.txt'
path2vec_test_dataset_fpath = 'FB15k237_path2vec_test_ready.txt'
vocab_file_path = 'FB15k237_vocab.txt'
max_length=0

reader = codecs.open(fb15k_train_file_path, 'r', encoding='utf-8')
writer = codecs.open(path2vec_dataset_fpath, 'w', encoding='utf-8')
writer2 = codecs.open(vocab_file_path, 'w', encoding='utf-8')
writer3 = codecs.open(path2vec_test_dataset_fpath, 'w', encoding='utf-8')

for line in reader:
    line = line.lstrip().rstrip()
    if line:
        elements = line.split('\t')
        if len(elements) ==3:
            entity1 = elements[0].lower().lstrip().rstrip()
            entity2 = elements[2].lower().lstrip().rstrip()
            
            G.add_edge(entity1, entity2)

reader.close()

reader = codecs.open(fb15k_test_file_path, 'r', encoding='utf-8')
for line in reader:
    line = line.lstrip().rstrip()
    if line:
        elements = line.split('\t')
        if len(elements) ==3:
            entity1 = elements[0].lower().lstrip().rstrip()
            entity2 = elements[2].lower().lstrip().rstrip()
            
            G.add_edge(entity1, entity2)

reader.close()

print('Is graph connected: ', nx.is_connected(G))
print('Number of connected components: ', len(list(nx.connected_components(G))))
cmp_lengths = [len(c) for c in sorted(nx.connected_components(G), key=len, reverse=True)]
print('Sizes of the connected components in descending order is: ', cmp_lengths)

print('Removing small components...')
components = [c for c in sorted(nx.connected_components(G), key=len)]
for idx in range(len(cmp_lengths)-1):
    component = components[idx]
    for v in component:
        G.remove_node(v)
        
print('Done removing small components.')
print('Is graph connected: ', nx.is_connected(G))
print('Number of connected components: ', len(list(nx.connected_components(G))))
cmp_lengths = [len(c) for c in sorted(nx.connected_components(G), key=len, reverse=True)]
print('Sizes of the connected components in descending order is: ', cmp_lengths)
print('')

degrees_sorted = [ d[1] for d in sorted(G.degree, key=lambda x: x[1], reverse=True)]
print('Max degree in the graph: ', degrees_sorted[0])
print('Min degree in the graph: ', degrees_sorted[-1])
print('Average degree in the graph: ', round(sum(degrees_sorted) / len(degrees_sorted)))
print('Median degree in the graph: ', int(statistics.median(degrees_sorted)))
#print('Graph diameter (greatest shortest path) is: %d'%nx.diameter(G))
print('')

graph_nodes = list(G.nodes)
i = 0
print('Selecting 1000 random pairs for test set...')
test_pairs = []
while i<1000:
    node = random.choice(graph_nodes)
    shps_for_node = dict(nx.shortest_path_length(G, source=node))
    if len(list(shps_for_node.keys())) > 5:
        other_node = random.choice(list(shps_for_node.keys()))
        line = node + '\t' + other_node + '\t' + str((1/(shps_for_node[other_node]+1))) + '\n'
        writer3.write(line)
        test_pairs.append(node+other_node)
        test_pairs.append(other_node+node)
        i += 1

writer3.close()
print('Done selecting random pairs for test set.')

final_graph = nx.Graph()

print('Persisting training set now...')
for source in G.nodes:
    vocab = source + '\n'
    writer2.write(vocab)
    shp_per_source = dict(nx.shortest_path_length(G, source=source))
    for target in shp_per_source:
        if source+target not in test_pairs:
            final_graph.add_edge(source, target, weight=float((1/(shp_per_source[target]+1))))
            if shp_per_source[target] > max_length:
                max_length = shp_per_source[target]
        
for edge in final_graph.edges.data('weight'):
    line = edge[0] + '\t' + edge[1] + '\t' + str(edge[2]) + '\n'
    writer.write(line)

        
print('Done persisting training set.')
print('Graph diameter (greatest shortest path) is: %d'%max_length)
writer.close()
writer2.close()




